unit Helps;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, Menus, Registry, Desc, Msg, ShellApi;

type
  THelpDlg = class(TForm)
    LB: TListBox;
    pm: TPopupMenu;
    AddItem: TMenuItem;
    DelItem: TMenuItem;
    ModifyItem: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AddItemClick(Sender: TObject);
    procedure pmPopup(Sender: TObject);
    procedure ModifyItemClick(Sender: TObject);
    procedure DelItemClick(Sender: TObject);
    procedure LBDblClick(Sender: TObject);
    procedure LBKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    Files: TStringList;
    Reg: TRegIniFile;
    procedure ExecuteFile;
    procedure WriteList(Ident: String; ls: TStrings);
  public
    { Public declarations }
  end;

procedure ExecuteHelps;

var
  HelpDlg: THelpDlg;

implementation

{$R *.DFM}

procedure ExecuteHelps;
Begin
  IF HelpDlg=Nil then
    HelpDlg:=THelpDlg.Create(Nil);
  HelpDlg.ShowModal;
End;

procedure THelpDlg.WriteList(Ident: String; ls: TStrings);
var
  i: Integer;
Begin
  Reg.EraseSection(Ident);
  For i:=0 To ls.Count-1 Do
    Reg.WriteString(Ident,ls[i],'');
End;

procedure THelpDlg.FormCreate(Sender: TObject);
begin
  Reg:=TRegIniFile.Create('Software\Borland\Delphi\3.0\Experts\Helps');
  Files := TStringList.Create;
  Reg.ReadSection('Description',Lb.Items);
  Reg.ReadSection('Files',Files);
  IF Lb.Items.Count>0 Then
    LB.ItemIndex := 0;
end;

procedure THelpDlg.FormDestroy(Sender: TObject);
begin
  WriteList('Description',Lb.Items);
  WriteList('Files',Files);
  Reg.Free;
  Files.Free;
end;

procedure THelpDlg.AddItemClick(Sender: TObject);
begin
  Application.CreateForm(TDescDlg,DescDlg);
  IF DescDlg.ShowModal=mrOK Then
  begin
    Lb.Items.Add(DescDlg.edDesc.Text);
    Files.Add(DescDlg.edFile.Text);
  end;
  DescDlg.Free;
end;

procedure THelpDlg.pmPopup(Sender: TObject);
begin
  ModifyItem.Enabled := Lb.ItemIndex>-1;
  DelItem.Enabled := Lb.ItemIndex>-1;
end;

procedure THelpDlg.ModifyItemClick(Sender: TObject);
begin
  Application.CreateForm(TDescDlg,DescDlg);
  With DescDlg do
  begin
    edFile.Text := Files[Lb.ItemIndex];
    edDesc.Text := Lb.Items[Lb.ItemIndex];
    IF ShowModal=mrOK Then
    begin
      Lb.Items[Lb.ItemIndex] := edDesc.Text;
      Files[Lb.ItemIndex] := edFile.Text;
    end;
    Free;
  end;
end;

procedure THelpDlg.DelItemClick(Sender: TObject);
begin
  IF MsgDlg('     ?',mtWarning,[mbYes,mbNo],0)=mrYes Then
  begin
    Files.Delete(Lb.ItemIndex);
    Lb.Items.Delete(Lb.ItemIndex);
  end;
end;

procedure THelpDlg.ExecuteFile;
var
  zParams: array[0..79] of Char;
begin
  IF LB.ItemIndex<0 Then Exit;
  ShellExecute(Handle, nil,
    StrPCopy(zParams, Files[Lb.ItemIndex]),'' , '', SW_SHOW);
  Close;
end;

procedure THelpDlg.LBDblClick(Sender: TObject);
begin
  ExecuteFile;
end;

procedure THelpDlg.LBKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  IF (Key=VK_RETURN) and (Shift=[]) Then
    ExecuteFile;
end;

procedure THelpDlg.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  IF Key=VK_ESCAPE Then
    Close;
end;

end.
