{*******************************************************}
{                                                       }
{         Delphi eXpert Extensions   (X)                }
{                                                       }
{         Copyright (c) 1997 Alexander Buzaev           }
{         FidoNet 2:5020/880.11@fidonet.org             }
{                                                       }
{*******************************************************}
{$DEFINE xExpert}

unit MNavExp;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics,
  Controls, Forms, Dialogs, StdCtrls, ParsPas, ComExp, Registry,
  ExptIntf, VirtIntf, DsgnIntf, ToolIntf, EditIntf, Menus, ExtCtrls,
  xCommon;

type
  TMNavExpDlg = class(TForm)
    pm: TPopupMenu;
    StayOnTop1: TMenuItem;
    plSearch: TPanel;
    plEd: TPanel;
    plList: TPanel;
    lbMethod: TListBox;
    cbFilter: TComboBox;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure lbMethodDblClick(Sender: TObject);
    procedure StayOnTop1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbFilterChange(Sender: TObject);
    procedure cbFilterKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure plEdResize(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure lbMethodKeyPress(Sender: TObject; var Key: Char);
  private
    Methods: TMethodList;
    Reg: TRegistry;
  public
  end;

var
  MNavExpDlg: TMNavExpDlg;

procedure ExecuteMNav;

implementation

{$IFNDEF xExpert}
  Uses MNavFm, EditUtil;
{$ENDIF}

{$R *.DFM}

procedure ExecuteMNav;
Begin
  MNavExpDlg:=TMNavExpDlg.Create(Nil);
  MNavExpDlg.ShowModal;
End;

procedure TMNavExpDlg.FormCreate(Sender: TObject);
begin
  ReadFormPlacement(Self, xKey);
  Reg:=TRegistry.Create;
  With Reg Do
  begin
    RootKey:=HKEY_CURRENT_USER;
    OpenKey(xKey+'Method Navigator', True);
  end;
  ReadList(Reg, 'cbFilter', cbFilter.Items);
  Methods:=TMethodList.Create;
{$IFDEF xExpert}
  Methods.Parse(GetCurrentFile);
  cbFilter.Text:=GetWordAtCursor;
{$ELSE}
  Methods.Parse(MNavForm.Editor.Lines);
  cbFilter.Text:='';//GetWordAtCursor;
{$ENDIF}
  cbFilterChange(Nil);
  IF lbMethod.Items.Count=0 Then
  begin
    cbFilter.Text:='';
    cbFilterChange(Nil);
  end;
end;

procedure TMNavExpDlg.FormClose(Sender: TObject;
  var Action: TCloseAction);
var
  i: Integer;
begin
  For i:=cbFilter.Items.Count-1 DownTo 16 Do
    cbFilter.Items.Delete(i);
  WriteList(Reg, 'cbFilter', cbFilter.Items);
  WriteFormPlacement(Self, xKey);
  Reg.Free;
  Methods.Free;
  Action:=caFree;  MNavExpDlg:=Nil;
end;

procedure TMNavExpDlg.StayOnTop1Click(Sender: TObject);
begin
 StayOnTop1.Checked:=Not StayOnTop1.Checked;
 if StayOnTop1.Checked then
   FormStyle:= fsStayOnTop
 else
   FormStyle:= fsNormal;
end;

procedure TMNavExpDlg.cbFilterChange(Sender: TObject);
var
  iIndex: Integer;
begin
  With lbMethod Do
  begin
    iIndex:=ItemIndex;
    Items.Assign(Methods.GetMethods(cbFilter.Text));
    IF Items.Count<iIndex Then
      ItemIndex:=iIndex
    Else
      ItemIndex:=0;
  end;
end;

procedure TMNavExpDlg.cbFilterKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  IF (Shift=[]) and (not cbFilter.DroppedDown) Then
  Case Key OF
    VK_DOWN:
    begin
      With lbMethod Do
      IF Items.Count>ItemIndex+1 Then
        ItemIndex:=ItemIndex+1;
      Key:=0;
    end;
    VK_UP:
    begin
      With lbMethod Do
      IF ItemIndex>0 Then
        ItemIndex:=ItemIndex-1;
      Key:=0;
    end;
    VK_PRIOR:
    begin
      lbMethod.ItemIndex:=0;
      Key:=0;
    end;
    VK_NEXT:
    begin
      lbMethod.ItemIndex:=lbMethod.Items.Count-1;
      Key:=0;
    end;
  End;//Case
end;

procedure TMNavExpDlg.plEdResize(Sender: TObject);
begin
  cbFilter.Width:=plEd.Width-4;
end;

procedure TMNavExpDlg.lbMethodDblClick(Sender: TObject);
var
  CurPos, TopPos: TEditPos;
begin
  IF Methods.GetMethodPos(TMethod(lbMethod.Items.Objects[lbMethod.ItemIndex]),
        TopPos, CurPos) Then
  begin
    cbFilter.Items.Insert(0,cbFilter.Text);
{$IFDEF xExpert}
    SetCursorPos(TopPos, CurPos);
{$ELSE}
    With MNavForm.Editor Do
    begin
      SelStart:=EditPosToPos(CurPos, Lines);
      SendMessage(Handle, EM_SCROLLCARET, 0, 0);
    end;
{$ENDIF}
  end;
  Close;
end;

procedure TMNavExpDlg.FormKeyPress(Sender: TObject; var Key: Char);
begin
  IF Key=#13 Then
  begin
    IF (ActiveControl=cbFilter) or (ActiveControl=lbMethod) Then
      lbMethodDblClick(nil);
    Key:=#0;
  end
  Else IF Key=#27 Then
  begin
    IF cbFilter.DroppedDown Then
      cbFilter.DroppedDown:=False
    Else
      Close;
    Key:=#0;
  end
end;

procedure TMNavExpDlg.lbMethodKeyPress(Sender: TObject; var Key: Char);
begin
  cbKeyChar(Key, cbFilter);
  With cbFilter Do
  begin
    SetFocus;
    SelLength:=0;
    SelStart:=Length(Text);
  end;
end;


end.
